package javax.swing.text;

import java.awt.Graphics;
import java.io.Serializable;
import java.awt.event.FocusEvent;
import javax.swing.event.ChangeEvent;
import java.awt.event.MouseListener;
import java.awt.event.FocusListener;
import javax.swing.event.ChangeListener;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.im.InputContext;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Clipboard;
import javax.swing.plaf.UIResource;
import javax.accessibility.AccessibleTextSequence;
import java.text.BreakIterator;
import java.awt.Shape;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.swing.event.DocumentEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentAdapter;
import java.awt.IllegalComponentStateException;
import javax.accessibility.AccessibleExtendedText;
import javax.accessibility.AccessibleEditableText;
import javax.accessibility.AccessibleAction;
import javax.accessibility.AccessibleText;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.swing.SwingAccessor;
import java.awt.font.TextHitInfo;
import javax.swing.KeyStroke;
import java.text.AttributedString;
import java.text.AttributedCharacterIterator;
import java.awt.event.KeyEvent;
import java.awt.event.InputMethodListener;
import java.awt.event.InputMethodEvent;
import java.awt.AWTEvent;
import java.awt.event.InputEvent;
import java.io.ObjectInputStream;
import javax.accessibility.AccessibleContext;
import sun.swing.text.TextComponentPrintable;
import java.awt.print.Printable;
import java.awt.print.PrinterAbortException;
import java.util.concurrent.ExecutionException;
import sun.swing.SwingUtilities2;
import java.util.concurrent.FutureTask;
import java.util.concurrent.Callable;
import javax.print.attribute.HashPrintRequestAttributeSet;
import sun.swing.PrintingStatus;
import java.awt.print.PrinterJob;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.PrintService;
import java.text.MessageFormat;
import java.awt.print.PrinterException;
import java.awt.Container;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.io.Writer;
import java.io.IOException;
import java.io.Reader;
import java.awt.event.ActionEvent;
import java.awt.EventQueue;
import java.awt.Rectangle;
import java.util.Hashtable;
import sun.awt.AppContext;
import java.util.HashMap;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.TransferHandler;
import java.awt.Point;
import java.awt.HeadlessException;
import java.awt.GraphicsEnvironment;
import java.beans.Transient;
import javax.swing.Action;
import java.awt.ComponentOrientation;
import java.awt.font.TextAttribute;
import javax.swing.event.DocumentListener;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.UIManager;
import javax.swing.plaf.TextUI;
import java.awt.im.InputMethodRequests;
import com.sun.beans.util.Cache;
import javax.swing.DropMode;
import java.awt.Insets;
import java.awt.Color;
import javax.accessibility.Accessible;
import javax.swing.Scrollable;
import javax.swing.JComponent;

public abstract class JTextComponent extends JComponent implements Scrollable, Accessible
{
    public static final String FOCUS_ACCELERATOR_KEY = "focusAcceleratorKey";
    private Document model;
    private transient Caret caret;
    private NavigationFilter navigationFilter;
    private transient Highlighter highlighter;
    private transient Keymap keymap;
    private transient JTextComponent.MutableCaretEvent caretEvent;
    private Color caretColor;
    private Color selectionColor;
    private Color selectedTextColor;
    private Color disabledTextColor;
    private boolean editable;
    private Insets margin;
    private char focusAccelerator;
    private boolean dragEnabled;
    private DropMode dropMode;
    private transient JTextComponent.DropLocation dropLocation;
    private static JTextComponent.DefaultTransferHandler defaultTransferHandler;
    private static Cache<Class<?>, Boolean> METHOD_OVERRIDDEN;
    private static final Object KEYMAP_TABLE;
    private transient InputMethodRequests inputMethodRequestsHandler;
    private SimpleAttributeSet composedTextAttribute;
    private String composedTextContent;
    private Position composedTextStart;
    private Position composedTextEnd;
    private Position latestCommittedTextStart;
    private Position latestCommittedTextEnd;
    private JTextComponent.ComposedTextCaret composedTextCaret;
    private transient Caret originalCaret;
    private boolean checkedInputOverride;
    private boolean needToSendKeyTypedEvent;
    private static final Object FOCUSED_COMPONENT;
    public static final String DEFAULT_KEYMAP = "default";
    
    public JTextComponent() {
        this.dropMode = DropMode.USE_SELECTION;
        this.enableEvents(2056L);
        this.addMouseListener(this.caretEvent = new JTextComponent.MutableCaretEvent(this));
        this.addFocusListener(this.caretEvent);
        this.setEditable(true);
        this.setDragEnabled(false);
        this.setLayout(null);
        this.updateUI();
    }
    
    @Override
    public TextUI getUI() {
        return (TextUI)this.ui;
    }
    
    public void setUI(TextUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public void updateUI() {
        this.setUI((TextUI)UIManager.getUI(this));
        this.invalidate();
    }
    
    public void addCaretListener(CaretListener l) {
        this.listenerList.add(CaretListener.class, l);
    }
    
    public void removeCaretListener(CaretListener l) {
        this.listenerList.remove(CaretListener.class, l);
    }
    
    public CaretListener[] getCaretListeners() {
        return (CaretListener[])this.listenerList.getListeners(CaretListener.class);
    }
    
    protected void fireCaretUpdate(CaretEvent caretEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == CaretListener.class) {
                ((CaretListener)listenerList[i + 1]).caretUpdate(caretEvent);
            }
        }
    }
    
    public void setDocument(Document document) {
        Document model = this.model;
        try {
            if ((model instanceof AbstractDocument)) {
                ((AbstractDocument)model).readLock();
            }
            if (this.accessibleContext != null) {
                this.model.removeDocumentListener((JTextComponent.AccessibleJTextComponent)this.accessibleContext);
            }
            if (this.inputMethodRequestsHandler != null) {
                this.model.removeDocumentListener((DocumentListener)this.inputMethodRequestsHandler);
            }
            this.model = document;
            Boolean b = this.getComponentOrientation().isLeftToRight() ? TextAttribute.RUN_DIRECTION_LTR : TextAttribute.RUN_DIRECTION_RTL;
            if (b != document.getProperty(TextAttribute.RUN_DIRECTION)) {
                document.putProperty(TextAttribute.RUN_DIRECTION, b);
            }
            this.firePropertyChange("document", model, document);
        }
        finally {
            if ((model instanceof AbstractDocument)) {
                ((AbstractDocument)model).readUnlock();
            }
        }
        this.revalidate();
        this.repaint();
        if (this.accessibleContext != null) {
            this.model.addDocumentListener((JTextComponent.AccessibleJTextComponent)this.accessibleContext);
        }
        if (this.inputMethodRequestsHandler != null) {
            this.model.addDocumentListener((DocumentListener)this.inputMethodRequestsHandler);
        }
    }
    
    public Document getDocument() {
        return this.model;
    }
    
    @Override
    public void setComponentOrientation(ComponentOrientation componentOrientation) {
        Document document = this.getDocument();
        if (document != null) {
            document.putProperty(TextAttribute.RUN_DIRECTION, componentOrientation.isLeftToRight() ? TextAttribute.RUN_DIRECTION_LTR : TextAttribute.RUN_DIRECTION_RTL);
        }
        super.setComponentOrientation(componentOrientation);
    }
    
    public Action[] getActions() {
        return this.getUI().getEditorKit(this).getActions();
    }
    
    public void setMargin(Insets margin) {
        this.firePropertyChange("margin", this.margin, this.margin = margin);
        this.invalidate();
    }
    
    public Insets getMargin() {
        return this.margin;
    }
    
    public void setNavigationFilter(NavigationFilter navigationFilter) {
        this.navigationFilter = navigationFilter;
    }
    
    public NavigationFilter getNavigationFilter() {
        return this.navigationFilter;
    }
    
    @Transient
    public Caret getCaret() {
        return this.caret;
    }
    
    public void setCaret(Caret caret) {
        if (this.caret != null) {
            this.caret.removeChangeListener(this.caretEvent);
            this.caret.deinstall(this);
        }
        Caret caret2 = this.caret;
        this.caret = caret;
        if (this.caret != null) {
            this.caret.install(this);
            this.caret.addChangeListener(this.caretEvent);
        }
        this.firePropertyChange("caret", caret2, this.caret);
    }
    
    public Highlighter getHighlighter() {
        return this.highlighter;
    }
    
    public void setHighlighter(Highlighter highlighter) {
        if (this.highlighter != null) {
            this.highlighter.deinstall(this);
        }
        Highlighter highlighter2 = this.highlighter;
        this.highlighter = highlighter;
        if (this.highlighter != null) {
            this.highlighter.install(this);
        }
        this.firePropertyChange("highlighter", highlighter2, highlighter);
    }
    
    public void setKeymap(Keymap keymap) {
        Keymap keymap2 = this.keymap;
        this.firePropertyChange("keymap", keymap2, this.keymap = keymap);
        this.updateInputMap(keymap2, keymap);
    }
    
    public void setDragEnabled(boolean dragEnabled) {
        if (dragEnabled && GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        this.dragEnabled = dragEnabled;
        return;
    }
    
    public boolean getDragEnabled() {
        return this.dragEnabled;
    }
    
    public final void setDropMode(DropMode dropMode) {
        if (dropMode != null) {
            switch (dropMode) {
                case USE_SELECTION:
                case INSERT: {
                    this.dropMode = dropMode;
                    return;
                }
                default: {
                    break;
                }
            }
        }
        throw new IllegalArgumentException(dropMode + ": Unsupported drop mode for text");
    }
    
    public final DropMode getDropMode() {
        return this.dropMode;
    }
    
    @Override
    JTextComponent.DropLocation dropLocationForPoint(Point point) {
        Position.Bias[] array = { null };
        int viewToModel = this.getUI().viewToModel(this, point, array);
        if (array[0] == null) {
            array[0] = Position.Bias.Forward;
        }
        return new JTextComponent.DropLocation(point, viewToModel, array[0]);
    }
    
    @Override
    Object setDropLocation(TransferHandler.DropLocation dropLocation, Object o, boolean b) {
        Object value = null;
        JTextComponent.DropLocation dropLocation2 = (JTextComponent.DropLocation)dropLocation;
        if (this.dropMode == DropMode.USE_SELECTION) {
            if (dropLocation2 == null) {
                if (o != null) {
                    Object[] array = (Object[])(Object[])o;
                    if (!b) {
                        if ((this.caret instanceof DefaultCaret)) {
                            ((DefaultCaret)this.caret).setDot((int)array[0], (Position.Bias)array[3]);
                            ((DefaultCaret)this.caret).moveDot((int)array[1], (Position.Bias)array[4]);
                        }
                        else {
                            this.caret.setDot((int)array[0]);
                            this.caret.moveDot((int)array[1]);
                        }
                    }
                    this.caret.setVisible((boolean)array[2]);
                }
            }
            else {
                if (this.dropLocation == null) {
                    if ((this.caret instanceof DefaultCaret)) {
                        DefaultCaret defaultCaret = (DefaultCaret)this.caret;
                        value = new Object[] { Integer.valueOf(defaultCaret.getMark()), Integer.valueOf(defaultCaret.getDot()), Boolean.valueOf(defaultCaret.isActive()), defaultCaret.getMarkBias(), defaultCaret.getDotBias() };
                    }
                    else {
                        value = new Object[] { Integer.valueOf(this.caret.getMark()), Integer.valueOf(this.caret.getDot()), Boolean.valueOf(this.caret.isVisible()) };
                    }
                    this.caret.setVisible(true);
                }
                else {
                    value = o;
                }
                if ((this.caret instanceof DefaultCaret)) {
                    ((DefaultCaret)this.caret).setDot(dropLocation2.getIndex(), dropLocation2.getBias());
                }
                else {
                    this.caret.setDot(dropLocation2.getIndex());
                }
            }
        }
        else if (dropLocation2 == null) {
            if (o != null) {
                this.caret.setVisible((boolean)o);
            }
        }
        else if (this.dropLocation == null) {
            value = Boolean.valueOf((this.caret instanceof DefaultCaret) ? ((DefaultCaret)this.caret).isActive() : this.caret.isVisible());
            this.caret.setVisible(false);
        }
        else {
            value = o;
        }
        this.firePropertyChange("dropLocation", this.dropLocation, this.dropLocation = dropLocation2);
        return value;
    }
    
    public final JTextComponent.DropLocation getDropLocation() {
        return this.dropLocation;
    }
    
    void updateInputMap(Keymap keymap, Keymap keymap2) {
        InputMap inputMap;
        for (inputMap = (inputMap = this.getInputMap(0)); inputMap != null && !(inputMap instanceof JTextComponent.KeymapWrapper); , inputMap = inputMap.getParent()) {}
        if (inputMap != null) {
            if (keymap2 == null) {
                if (inputMap != inputMap) {
                    inputMap.setParent(inputMap.getParent());
                }
                else {
                    inputMap.setParent(null);
                }
            }
            else {
                JTextComponent.KeymapWrapper parent2 = new KeymapWrapper(keymap2);
                inputMap.setParent(parent2);
                if (inputMap != inputMap) {
                    parent2.setParent(inputMap.getParent());
                }
            }
        }
        else if (keymap2 != null) {
            InputMap inputMap2 = this.getInputMap(0);
            if (inputMap2 != null) {
                JTextComponent.KeymapWrapper parent = new KeymapWrapper(keymap2);
                parent.setParent(inputMap2.getParent());
                inputMap2.setParent(parent);
            }
        }
        ActionMap actionMap;
        for (actionMap = (actionMap = this.getActionMap()); actionMap != null && !(actionMap instanceof JTextComponent.KeymapActionMap); , actionMap = actionMap.getParent()) {}
        if (actionMap != null) {
            if (keymap2 == null) {
                if (actionMap != actionMap) {
                    actionMap.setParent(actionMap.getParent());
                }
                else {
                    actionMap.setParent(null);
                }
            }
            else {
                JTextComponent.KeymapActionMap parent4 = new KeymapActionMap(keymap2);
                actionMap.setParent(parent4);
                if (actionMap != actionMap) {
                    parent4.setParent(actionMap.getParent());
                }
            }
        }
        else if (keymap2 != null) {
            ActionMap actionMap2 = this.getActionMap();
            if (actionMap2 != null) {
                JTextComponent.KeymapActionMap parent3 = new KeymapActionMap(keymap2);
                parent3.setParent(actionMap2.getParent());
                actionMap2.setParent(parent3);
            }
        }
    }
    
    public Keymap getKeymap() {
        return this.keymap;
    }
    
    public static Keymap addKeymap(String key, Keymap keymap) {
        JTextComponent.DefaultKeymap value = new JTextComponent.DefaultKeymap(key, keymap);
        if (key != null) {
            getKeymapTable().put(key, value);
            return value;
        }
        return value;
    }
    
    public static Keymap removeKeymap(String key) {
        return (Keymap)getKeymapTable().remove(key);
    }
    
    public static Keymap getKeymap(String key) {
        return (Keymap)getKeymapTable().get(key);
    }
    
    private static HashMap<String, Keymap> getKeymapTable() {
        synchronized (JTextComponent.KEYMAP_TABLE) {
            AppContext appContext = AppContext.getAppContext();
            HashMap value = (HashMap)appContext.get(JTextComponent.KEYMAP_TABLE);
            if (value == null) {
                value = new HashMap(17);
                appContext.put(JTextComponent.KEYMAP_TABLE, value);
                addKeymap("default", null).setDefaultAction(new DefaultEditorKit.DefaultKeyTypedAction());
            }
            return value;
        }
    }
    
    public static void loadKeymap(Keymap keymap, JTextComponent.KeyBinding[] array, Action[] array2) {
        Hashtable<Object, Action> hashtable = new Hashtable();
        for (Action value : array2) {
            String s = (String)value.getValue("Name");
            hashtable.put((s != null) ? s : "", value);
        }
        for (JTextComponent.KeyBinding keyBinding : array) {
            Action action = (Action)hashtable.get(keyBinding.actionName);
            if (action != null) {
                keymap.addActionForKeyStroke(keyBinding.key, action);
            }
        }
    }
    
    public Color getCaretColor() {
        return this.caretColor;
    }
    
    public void setCaretColor(Color caretColor) {
        this.firePropertyChange("caretColor", this.caretColor, this.caretColor = caretColor);
    }
    
    public Color getSelectionColor() {
        return this.selectionColor;
    }
    
    public void setSelectionColor(Color selectionColor) {
        this.firePropertyChange("selectionColor", this.selectionColor, this.selectionColor = selectionColor);
    }
    
    public Color getSelectedTextColor() {
        return this.selectedTextColor;
    }
    
    public void setSelectedTextColor(Color selectedTextColor) {
        this.firePropertyChange("selectedTextColor", this.selectedTextColor, this.selectedTextColor = selectedTextColor);
    }
    
    public Color getDisabledTextColor() {
        return this.disabledTextColor;
    }
    
    public void setDisabledTextColor(Color disabledTextColor) {
        this.firePropertyChange("disabledTextColor", this.disabledTextColor, this.disabledTextColor = disabledTextColor);
    }
    
    public void replaceSelection(String s) {
        Document document = this.getDocument();
        if (document != null) {
            try {
                int saveComposedText = this.saveComposedText(this.caret.getDot());
                int min = Math.min(this.caret.getDot(), this.caret.getMark());
                int max = Math.max(this.caret.getDot(), this.caret.getMark());
                if ((document instanceof AbstractDocument)) {
                    ((AbstractDocument)document).replace(min, max - min, s, null);
                }
                else {
                    if (min != max) {
                        document.remove(min, max - min);
                    }
                    if (s != null && s.length() > 0) {
                        document.insertString(min, s, null);
                    }
                }
                if (!(saveComposedText == 0)) {
                    this.restoreComposedText();
                }
            }
            catch (BadLocationException ex) {
                UIManager.getLookAndFeel().provideErrorFeedback(this);
            }
        }
    }
    
    public String getText(int n, int n2) throws BadLocationException {
        return this.getDocument().getText(n, n2);
    }
    
    public Rectangle modelToView(int n) throws BadLocationException {
        return this.getUI().modelToView(this, n);
    }
    
    public int viewToModel(Point point) {
        return this.getUI().viewToModel(this, point);
    }
    
    public void cut() {
        if (this.isEditable() && this.isEnabled()) {
            this.invokeAction("cut", TransferHandler.getCutAction());
        }
    }
    
    public void copy() {
        this.invokeAction("copy", TransferHandler.getCopyAction());
    }
    
    public void paste() {
        if (this.isEditable() && this.isEnabled()) {
            this.invokeAction("paste", TransferHandler.getPasteAction());
        }
    }
    
    private void invokeAction(String key, Action action) {
        ActionMap actionMap = this.getActionMap();
        Action value = null;
        if (actionMap != null) {
            value = actionMap.get(key);
        }
        if (value == null) {
            this.installDefaultTransferHandlerIfNecessary();
            value = action;
        }
        value.actionPerformed(new ActionEvent(this, 1001, (String)value.getValue("Name"), EventQueue.getMostRecentEventTime(), this.getCurrentEventModifiers()));
    }
    
    private void installDefaultTransferHandlerIfNecessary() {
        if (this.getTransferHandler() == null) {
            if (JTextComponent.defaultTransferHandler == null) {
                JTextComponent.defaultTransferHandler = new JTextComponent.DefaultTransferHandler();
            }
            this.setTransferHandler(JTextComponent.defaultTransferHandler);
        }
    }
    
    public void moveCaretPosition(int i) {
        Document document = this.getDocument();
        if (document != null) {
            if (i > document.getLength() || i < 0) {
                throw new IllegalArgumentException("bad position: " + i);
            }
            this.caret.moveDot(i);
        }
    }
    
    public void setFocusAccelerator(char ch) {
        int upperCase = Character.toUpperCase(ch);
        int focusAccelerator = this.focusAccelerator;
        this.firePropertyChange("focusAcceleratorKey", (char)focusAccelerator, this.focusAccelerator = upperCase);
        this.firePropertyChange("focusAccelerator", (char)focusAccelerator, this.focusAccelerator);
    }
    
    public char getFocusAccelerator() {
        return this.focusAccelerator;
    }
    
    public void read(Reader reader, Object o) throws IOException {
        EditorKit editorKit = this.getUI().getEditorKit(this);
        Document defaultDocument = editorKit.createDefaultDocument();
        if (o != null) {
            defaultDocument.putProperty("stream", o);
        }
        try {
            editorKit.read(reader, defaultDocument, 0);
            this.setDocument(defaultDocument);
        }
        catch (BadLocationException ex) {
            throw new IOException(ex.getMessage());
        }
    }
    
    public void write(Writer writer) throws IOException {
        Document document = this.getDocument();
        try {
            this.getUI().getEditorKit(this).write(writer, document, 0, document.getLength());
        }
        catch (BadLocationException ex) {
            throw new IOException(ex.getMessage());
        }
    }
    
    @Override
    public void removeNotify() {
        super.removeNotify();
        if (getFocusedComponent() == this) {
            AppContext.getAppContext().remove(JTextComponent.FOCUSED_COMPONENT);
        }
    }
    
    public void setCaretPosition(int n) {
        Document document = this.getDocument();
        if (document != null) {
            if (n > document.getLength() || n < 0) {
                throw new IllegalArgumentException("bad position: " + n);
            }
            this.caret.setDot(n);
        }
    }
    
    @Transient
    public int getCaretPosition() {
        return this.caret.getDot();
    }
    
    public void setText(String s) {
        try {
            Document document = this.getDocument();
            if ((document instanceof AbstractDocument)) {
                ((AbstractDocument)document).replace(0, document.getLength(), s, null);
            }
            else {
                document.remove(0, document.getLength());
                document.insertString(0, s, null);
            }
        }
        catch (BadLocationException ex) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
        }
    }
    
    public String getText() {
        Document document = this.getDocument();
        String text;
        try {
            text = document.getText(0, document.getLength());
        }
        catch (BadLocationException ex) {
            text = null;
        }
        return text;
    }
    
    public String getSelectedText() {
        String text = null;
        int min = Math.min(this.caret.getDot(), this.caret.getMark());
        int max = Math.max(this.caret.getDot(), this.caret.getMark());
        if (min != max) {
            try {
                text = this.getDocument().getText(min, max - min);
            }
            catch (BadLocationException ex) {
                throw new IllegalArgumentException(ex.getMessage());
            }
        }
        return text;
    }
    
    public boolean isEditable() {
        return this.editable;
    }
    
    public void setEditable(boolean editable) {
        if (editable != this.editable) {
            int editable2 = this.editable;
            this.enableInputMethods(this.editable = editable);
            this.firePropertyChange("editable", Boolean.valueOf((boolean)editable2), Boolean.valueOf(this.editable));
            this.repaint();
        }
    }
    
    @Transient
    public int getSelectionStart() {
        return Math.min(this.caret.getDot(), this.caret.getMark());
    }
    
    public void setSelectionStart(int n) {
        this.select(n, this.getSelectionEnd());
    }
    
    @Transient
    public int getSelectionEnd() {
        return Math.max(this.caret.getDot(), this.caret.getMark());
    }
    
    public void setSelectionEnd(int n) {
        this.select(this.getSelectionStart(), n);
    }
    
    public void select(int caretPosition, int n) {
        int length = this.getDocument().getLength();
        if (caretPosition < 0) {
            caretPosition = 0;
        }
        if (caretPosition > length) {
            caretPosition = length;
        }
        if (n > length) {
            n = length;
        }
        if (n < caretPosition) {
            n = caretPosition;
        }
        this.setCaretPosition(caretPosition);
        this.moveCaretPosition(n);
    }
    
    public void selectAll() {
        Document document = this.getDocument();
        if (document != null) {
            this.setCaretPosition(0);
            this.moveCaretPosition(document.getLength());
        }
    }
    
    @Override
    public String getToolTipText(MouseEvent event) {
        String s = super.getToolTipText(event);
        if (s == null) {
            TextUI ui = this.getUI();
            if (ui != null) {
                s = ui.getToolTipText(this, new Point(event.getX(), event.getY()));
            }
        }
        return s;
    }
    
    @Override
    public Dimension getPreferredScrollableViewportSize() {
        return this.getPreferredSize();
    }
    
    @Override
    public int getScrollableUnitIncrement(Rectangle rectangle, int i, int n) {
        switch (i) {
            case 1: {
                return rectangle.height / 10;
            }
            case 0: {
                return rectangle.width / 10;
            }
            default: {
                throw new IllegalArgumentException("Invalid orientation: " + i);
            }
        }
    }
    
    @Override
    public int getScrollableBlockIncrement(Rectangle rectangle, int i, int n) {
        switch (i) {
            case 1: {
                return rectangle.height;
            }
            case 0: {
                return rectangle.width;
            }
            default: {
                throw new IllegalArgumentException("Invalid orientation: " + i);
            }
        }
    }
    
    @Override
    public boolean getScrollableTracksViewportWidth() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return unwrappedParent instanceof JViewport && unwrappedParent.getWidth() > this.getPreferredSize().width;
    }
    
    @Override
    public boolean getScrollableTracksViewportHeight() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return unwrappedParent instanceof JViewport && unwrappedParent.getHeight() > this.getPreferredSize().height;
    }
    
    public boolean print() throws PrinterException {
        return this.print(null, null, true, null, null, true);
    }
    
    public boolean print(MessageFormat messageFormat, MessageFormat messageFormat2) throws PrinterException {
        return this.print(messageFormat, messageFormat2, true, null, null, true);
    }
    
    public boolean print(MessageFormat messageFormat, MessageFormat messageFormat2, boolean b, PrintService printService, final PrintRequestAttributeSet attributes, boolean b2) throws PrinterException {
        final PrinterJob printerJob = PrinterJob.getPrinterJob();
        int headless = GraphicsEnvironment.isHeadless();
        final int eventDispatchThread = SwingUtilities.isEventDispatchThread();
        Printable printable = this.getPrintable(messageFormat, messageFormat2);
        final PrintingStatus printingStatus;
        Printable notificationPrintable;
        if (b2 && headless == 0) {
            printingStatus = PrintingStatus.createPrintingStatus(this, printerJob);
            notificationPrintable = printingStatus.createNotificationPrintable(printable);
        }
        else {
            printingStatus = null;
            notificationPrintable = printable;
        }
        if (printService != null) {
            printerJob.setPrintService(printService);
        }
        printerJob.setPrintable(notificationPrintable);
        attributes = ((attributes == null) ? new HashPrintRequestAttributeSet() : attributes);
        if (b && headless == 0 && !printerJob.printDialog(attributes)) {
            return false;
        }
        final Object o = new FutureTask(new Callable<Object>() {
            @Override
            public Object call() throws Exception {
                try {
                    printerJob.print(attributes);
                }
                finally {
                    if (printingStatus != null) {
                        printingStatus.dispose();
                    }
                }
                return null;
            }
        });
        JTextComponent.3 task = new Runnable() {
            @Override
            public void run() {
                boolean booleanValue = false;
                if (!(eventDispatchThread == 0)) {
                    if (JTextComponent.this.isEnabled()) {
                        booleanValue = true;
                        JTextComponent.this.setEnabled(false);
                    }
                }
                else {
                    try {
                        booleanValue = (boolean)SwingUtilities2.submit(new Callable<Boolean>() {
                            @Override
                            public Boolean call() throws Exception {
                                int enabled = JTextComponent.this.isEnabled();
                                if (!(enabled == 0)) {
                                    JTextComponent.this.setEnabled(false);
                                }
                                return Boolean.valueOf((boolean)enabled);
                            }
                        }).get();
                    }
                    catch (InterruptedException cause) {
                        throw new RuntimeException(cause);
                    }
                    catch (ExecutionException ex) {
                        Throwable cause2 = ex.getCause();
                        if ((cause2 instanceof Error)) {
                            throw (Error)cause2;
                        }
                        if ((cause2 instanceof RuntimeException)) {
                            throw (RuntimeException)cause2;
                        }
                        throw new AssertionError(cause2);
                    }
                }
                JTextComponent.this.getDocument().render(o);
                if (!(booleanValue == 0)) {
                    if (!(eventDispatchThread == 0)) {
                        JTextComponent.this.setEnabled(true);
                    }
                    else {
                        try {
                            SwingUtilities2.submit(new Runnable() {
                                @Override
                                public void run() {
                                    JTextComponent.this.setEnabled(true);
                                }
                            }, null).get();
                        }
                        catch (InterruptedException cause3) {
                            throw new RuntimeException(cause3);
                        }
                        catch (ExecutionException ex2) {
                            Throwable cause4 = ex2.getCause();
                            if ((cause4 instanceof Error)) {
                                throw (Error)cause4;
                            }
                            if ((cause4 instanceof RuntimeException)) {
                                throw (RuntimeException)cause4;
                            }
                            throw new AssertionError(cause4);
                        }
                    }
                }
            }
        };
        if (!b2 || headless) {
            task.run();
        }
        else if (!(eventDispatchThread == 0)) {
            new Thread(task).start();
            printingStatus.showModal(true);
        }
        else {
            printingStatus.showModal(false);
            task.run();
        }
        try {
            o.get();
        }
        catch (InterruptedException cause) {
            throw new RuntimeException(cause);
        }
        catch (ExecutionException ex) {
            Throwable cause2 = ex.getCause();
            if ((cause2 instanceof PrinterAbortException)) {
                if (printingStatus != null && printingStatus.isAborted()) {
                    return false;
                }
                throw (PrinterAbortException)cause2;
            }
            else {
                if ((cause2 instanceof PrinterException)) {
                    throw (PrinterException)cause2;
                }
                if ((cause2 instanceof RuntimeException)) {
                    throw (RuntimeException)cause2;
                }
                if ((cause2 instanceof Error)) {
                    throw (Error)cause2;
                }
                throw new AssertionError(cause2);
            }
        }
        return true;
    }
    
    public Printable getPrintable(MessageFormat headerFormat, MessageFormat footerFormat) {
        return TextComponentPrintable.getPrintable(this, headerFormat, footerFormat);
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JTextComponent.AccessibleJTextComponent();
        }
        return this.accessibleContext;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.addMouseListener(this.caretEvent = new JTextComponent.MutableCaretEvent(this));
        this.addFocusListener(this.caretEvent);
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",caretColor=" + ((this.caretColor != null) ? this.caretColor.toString() : "") + ",disabledTextColor=" + ((this.disabledTextColor != null) ? this.disabledTextColor.toString() : "") + ",editable=" + (this.editable ? "true" : "false") + ",margin=" + ((this.margin != null) ? this.margin.toString() : "") + ",selectedTextColor=" + ((this.selectedTextColor != null) ? this.selectedTextColor.toString() : "") + ",selectionColor=" + ((this.selectionColor != null) ? this.selectionColor.toString() : "");
    }
    
    static final JTextComponent getFocusedComponent() {
        return (JTextComponent)AppContext.getAppContext().get(JTextComponent.FOCUSED_COMPONENT);
    }
    
    private int getCurrentEventModifiers() {
        int n = 0;
        AWTEvent currentEvent = EventQueue.getCurrentEvent();
        if ((currentEvent instanceof InputEvent)) {
            n = ((InputEvent)currentEvent).getModifiers();
        }
        else if ((currentEvent instanceof ActionEvent)) {
            n = ((ActionEvent)currentEvent).getModifiers();
        }
        return n;
    }
    
    @Override
    protected void processInputMethodEvent(InputMethodEvent inputMethodEvent) {
        super.processInputMethodEvent(inputMethodEvent);
        if (!inputMethodEvent.isConsumed()) {
            if (!this.isEditable()) {
                return;
            }
            switch (inputMethodEvent.getID()) {
                case 1100: {
                    this.replaceInputMethodText(inputMethodEvent);
                }
                case 1101: {
                    this.setInputMethodCaretPosition(inputMethodEvent);
                    break;
                }
            }
            inputMethodEvent.consume();
        }
    }
    
    @Override
    public InputMethodRequests getInputMethodRequests() {
        if (this.inputMethodRequestsHandler == null) {
            this.inputMethodRequestsHandler = new JTextComponent.InputMethodRequestsHandler();
            Document document = this.getDocument();
            if (document != null) {
                document.addDocumentListener((DocumentListener)this.inputMethodRequestsHandler);
            }
        }
        return this.inputMethodRequestsHandler;
    }
    
    @Override
    public void addInputMethodListener(InputMethodListener l) {
        super.addInputMethodListener(l);
        if (l != null) {
            this.needToSendKeyTypedEvent = false;
            this.checkedInputOverride = true;
        }
    }
    
    private void replaceInputMethodText(InputMethodEvent inputMethodEvent) {
        int i = inputMethodEvent.getCommittedCharacterCount();
        AttributedCharacterIterator text = inputMethodEvent.getText();
        Document document = this.getDocument();
        if (this.composedTextExists()) {
            try {
                document.remove(this.composedTextStart.getOffset(), this.composedTextEnd.getOffset() - this.composedTextStart.getOffset());
            }
            catch (BadLocationException ex) {}
            Position position = null;
            this.composedTextEnd = position;
            this.composedTextStart = position;
            this.composedTextAttribute = null;
            this.composedTextContent = null;
        }
        if (text != null) {
            text.first();
            int dot = 0;
            int dot2 = 0;
            if (i > 0) {
                dot = this.caret.getDot();
                if (this.shouldSynthensizeKeyEvents()) {
                    int keyChar = text.current();
                    while (i > 0) {
                        this.processKeyEvent(new KeyEvent(this, 400, EventQueue.getMostRecentEventTime(), 0, 0, (char)keyChar));
                        keyChar = text.next();
                        --i;
                    }
                }
                else {
                    StringBuilder sb = new StringBuilder();
                    int c = text.current();
                    while (i > 0) {
                        sb.append((char)c);
                        c = text.next();
                        --i;
                    }
                    this.mapCommittedTextToAction(sb.toString());
                }
                dot2 = this.caret.getDot();
            }
            int index = text.getIndex();
            if (index < text.getEndIndex()) {
                this.createComposedTextAttribute(index, text);
                try {
                    this.replaceSelection(null);
                    document.insertString(this.caret.getDot(), this.composedTextContent, this.composedTextAttribute);
                    this.composedTextStart = document.createPosition(this.caret.getDot() - this.composedTextContent.length());
                    this.composedTextEnd = document.createPosition(this.caret.getDot());
                }
                catch (BadLocationException ex2) {
                    Position position2 = null;
                    this.composedTextEnd = position2;
                    this.composedTextStart = position2;
                    this.composedTextAttribute = null;
                    this.composedTextContent = null;
                }
            }
            if (dot != dot2) {
                try {
                    this.latestCommittedTextStart = document.createPosition(dot);
                    this.latestCommittedTextEnd = document.createPosition(dot2);
                }
                catch (BadLocationException ex3) {
                    Position position4 = null;
                    this.latestCommittedTextEnd = position4;
                    this.latestCommittedTextStart = position4;
                }
            }
            else {
                Position position3 = null;
                this.latestCommittedTextEnd = position3;
                this.latestCommittedTextStart = position3;
            }
        }
    }
    
    private void createComposedTextAttribute(int n, AttributedCharacterIterator text) {
        this.getDocument();
        StringBuilder sb = new StringBuilder();
        for (int i = text.setIndex(n); i != '\uffff'; i = text.next()) {
            sb.append((char)i);
        }
        this.composedTextContent = sb.toString();
        (this.composedTextAttribute = new SimpleAttributeSet()).addAttribute(StyleConstants.ComposedTextAttribute, new AttributedString(text, n, text.getEndIndex()));
    }
    
    protected boolean saveComposedText(int n) {
        if (this.composedTextExists()) {
            int offset = this.composedTextStart.getOffset();
            int n2 = this.composedTextEnd.getOffset() - this.composedTextStart.getOffset();
            if (n >= offset && n <= offset + n2) {
                try {
                    this.getDocument().remove(offset, n2);
                    return true;
                }
                catch (BadLocationException ex) {}
            }
        }
        return false;
    }
    
    protected void restoreComposedText() {
        Document document = this.getDocument();
        try {
            document.insertString(this.caret.getDot(), this.composedTextContent, this.composedTextAttribute);
            this.composedTextStart = document.createPosition(this.caret.getDot() - this.composedTextContent.length());
            this.composedTextEnd = document.createPosition(this.caret.getDot());
        }
        catch (BadLocationException ex) {}
    }
    
    private void mapCommittedTextToAction(String command) {
        Keymap keymap = this.getKeymap();
        if (keymap != null) {
            Action action = null;
            if (command.length() == 1) {
                action = keymap.getAction(KeyStroke.getKeyStroke(command.charAt(0)));
            }
            if (action == null) {
                action = keymap.getDefaultAction();
            }
            if (action != null) {
                action.actionPerformed(new ActionEvent(this, 1001, command, EventQueue.getMostRecentEventTime(), this.getCurrentEventModifiers()));
            }
        }
    }
    
    private void setInputMethodCaretPosition(InputMethodEvent inputMethodEvent) {
        if (this.composedTextExists()) {
            int offset = this.composedTextStart.getOffset();
            if (!(this.caret instanceof JTextComponent.ComposedTextCaret)) {
                if (this.composedTextCaret == null) {
                    this.composedTextCaret = new JTextComponent.ComposedTextCaret();
                }
                this.exchangeCaret(this.originalCaret = this.caret, this.composedTextCaret);
            }
            TextHitInfo caret = inputMethodEvent.getCaret();
            if (caret != null) {
                int insertionIndex = caret.getInsertionIndex();
                offset += insertionIndex;
                if (insertionIndex == 0) {
                    try {
                        Rectangle modelToView = this.modelToView(offset);
                        modelToView.x += Math.min(this.modelToView(this.composedTextEnd.getOffset()).x - modelToView.x, this.getBounds().width);
                        this.scrollRectToVisible(modelToView);
                    }
                    catch (BadLocationException ex) {}
                }
            }
            this.caret.setDot(offset);
        }
        else if ((this.caret instanceof JTextComponent.ComposedTextCaret)) {
            int dot = this.caret.getDot();
            this.exchangeCaret(this.caret, this.originalCaret);
            this.caret.setDot(dot);
        }
    }
    
    private void exchangeCaret(Caret caret, Caret caret2) {
        int blinkRate = caret.getBlinkRate();
        this.setCaret(caret2);
        this.caret.setBlinkRate(blinkRate);
        this.caret.setVisible(this.hasFocus());
    }
    
    private boolean shouldSynthensizeKeyEvents() {
        if (!this.checkedInputOverride) {
            this.needToSendKeyTypedEvent = !(boolean)JTextComponent.METHOD_OVERRIDDEN.get(this.getClass());
            this.checkedInputOverride = true;
        }
        return this.needToSendKeyTypedEvent;
    }
    
    boolean composedTextExists() {
        return this.composedTextStart != null;
    }
    
    static int access$300(JTextComponent textComponent) {
        return textComponent.getCurrentEventModifiers();
    }
    
    static {
        SwingAccessor.setJTextComponentAccessor(new SwingAccessor.JTextComponentAccessor() {
            @Override
            public TransferHandler.DropLocation dropLocationForPoint(JTextComponent textComponent, Point point) {
                return textComponent.dropLocationForPoint(point);
            }
            
            @Override
            public Object setDropLocation(JTextComponent textComponent, TransferHandler.DropLocation dropLocation, Object o, boolean b) {
                return textComponent.setDropLocation(dropLocation, o, b);
            }
        });
        JTextComponent.METHOD_OVERRIDDEN = new Cache<Class<?>, Boolean>(Cache.Kind.WEAK, Cache.Kind.STRONG) {
            @Override
            public Boolean create(Class<?> clazz) {
                if (JTextComponent.class == clazz) {
                    return Boolean.FALSE;
                }
                if ((boolean)this.get(clazz.getSuperclass())) {
                    return Boolean.TRUE;
                }
                return (Boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
                    @Override
                    public Boolean run() {
                        try {
                            clazz.getDeclaredMethod("processInputMethodEvent", new Class[] { InputMethodEvent.class });
                            return Boolean.TRUE;
                        }
                        catch (NoSuchMethodException ex) {
                            return Boolean.FALSE;
                        }
                    }
                });
            }
        };
        KEYMAP_TABLE = new StringBuilder("JTextComponent_KeymapTable");
        FOCUSED_COMPONENT = new StringBuilder("JTextComponent_FocusedComponent");
    }
    
    public static class KeyBinding
    {
        public KeyStroke key;
        public String actionName;
        
        public KeyBinding(KeyStroke key, String actionName) {
            this.key = key;
            this.actionName = actionName;
        }
    }
    
    public class AccessibleJTextComponent extends JComponent.AccessibleJComponent implements AccessibleText, CaretListener, DocumentListener, AccessibleAction, AccessibleEditableText, AccessibleExtendedText
    {
        int caretPos;
        Point oldLocationOnScreen;
        
        public AccessibleJTextComponent() {
            Document document = JTextComponent.this.getDocument();
            if (document != null) {
                document.addDocumentListener(this);
            }
            JTextComponent.this.addCaretListener(this);
            this.caretPos = this.getCaretPosition();
            try {
                this.oldLocationOnScreen = this.getLocationOnScreen();
            }
            catch (IllegalComponentStateException ex) {}
            JTextComponent.this.addComponentListener(new ComponentAdapter() {
                @Override
                public void componentMoved(ComponentEvent componentEvent) {
                    try {
                        Point locationOnScreen = JTextComponent.AccessibleJTextComponent.this.getLocationOnScreen();
                        JTextComponent.AccessibleJTextComponent.this.firePropertyChange("AccessibleVisibleData", JTextComponent.AccessibleJTextComponent.this.oldLocationOnScreen, locationOnScreen);
                        JTextComponent.AccessibleJTextComponent.this.oldLocationOnScreen = locationOnScreen;
                    }
                    catch (IllegalComponentStateException ex) {}
                }
            });
        }
        
        @Override
        public void caretUpdate(CaretEvent caretEvent) {
            int dot = caretEvent.getDot();
            int mark = caretEvent.getMark();
            if (this.caretPos != dot) {
                this.firePropertyChange("AccessibleCaret", new Integer(this.caretPos), new Integer(dot));
                this.caretPos = dot;
                try {
                    this.oldLocationOnScreen = this.getLocationOnScreen();
                }
                catch (IllegalComponentStateException ex) {}
            }
            if (mark != dot) {
                this.firePropertyChange("AccessibleSelection", null, this.getSelectedText());
            }
        }
        
        @Override
        public void insertUpdate(DocumentEvent documentEvent) {
            Integer newValue = new Integer(documentEvent.getOffset());
            if (SwingUtilities.isEventDispatchThread()) {
                this.firePropertyChange("AccessibleText", null, newValue);
            }
            else {
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        JTextComponent.AccessibleJTextComponent.this.firePropertyChange("AccessibleText", null, newValue);
                    }
                });
            }
        }
        
        @Override
        public void removeUpdate(DocumentEvent documentEvent) {
            Integer newValue = new Integer(documentEvent.getOffset());
            if (SwingUtilities.isEventDispatchThread()) {
                this.firePropertyChange("AccessibleText", null, newValue);
            }
            else {
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        JTextComponent.AccessibleJTextComponent.this.firePropertyChange("AccessibleText", null, newValue);
                    }
                });
            }
        }
        
        @Override
        public void changedUpdate(DocumentEvent documentEvent) {
            Integer newValue = new Integer(documentEvent.getOffset());
            if (SwingUtilities.isEventDispatchThread()) {
                this.firePropertyChange("AccessibleText", null, newValue);
            }
            else {
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        JTextComponent.AccessibleJTextComponent.this.firePropertyChange("AccessibleText", null, newValue);
                    }
                });
            }
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (JTextComponent.this.isEditable()) {
                accessibleStateSet.add(AccessibleState.EDITABLE);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TEXT;
        }
        
        @Override
        public AccessibleText getAccessibleText() {
            return this;
        }
        
        @Override
        public int getIndexAtPoint(Point point) {
            if (point == null) {
                return -1;
            }
            return JTextComponent.this.viewToModel(point);
        }
        
        Rectangle getRootEditorRect() {
            Rectangle bounds = JTextComponent.this.getBounds();
            if (bounds.width > 0 && bounds.height > 0) {
                Rectangle rectangle = bounds;
                int n = 0;
                bounds.y = n;
                rectangle.x = n;
                Insets insets = JTextComponent.this.getInsets();
                bounds.x += insets.left;
                bounds.y += insets.top;
                bounds.width -= insets.left + insets.right;
                bounds.height -= insets.top + insets.bottom;
                return bounds;
            }
            return null;
        }
        
        @Override
        public Rectangle getCharacterBounds(int n) {
            if (n < 0 || n > JTextComponent.this.model.getLength() - 1) {
                return null;
            }
            TextUI ui = JTextComponent.this.getUI();
            if (ui == null) {
                return null;
            }
            Rectangle rectangle = null;
            Rectangle rootEditorRect = this.getRootEditorRect();
            if (rootEditorRect == null) {
                return null;
            }
            if ((JTextComponent.this.model instanceof AbstractDocument)) {
                ((AbstractDocument)JTextComponent.this.model).readLock();
            }
            try {
                View rootView = ui.getRootView(JTextComponent.this);
                if (rootView != null) {
                    rootView.setSize((float)rootEditorRect.width, (float)rootEditorRect.height);
                    Shape modelToView = rootView.modelToView(n, Position.Bias.Forward, n + 1, Position.Bias.Backward, rootEditorRect);
                    rectangle = ((modelToView instanceof Rectangle) ? ((Rectangle)modelToView) : modelToView.getBounds());
                }
            }
            catch (BadLocationException ex) {}
            finally {
                if ((JTextComponent.this.model instanceof AbstractDocument)) {
                    ((AbstractDocument)JTextComponent.this.model).readUnlock();
                }
            }
            return rectangle;
        }
        
        @Override
        public int getCharCount() {
            return JTextComponent.this.model.getLength();
        }
        
        @Override
        public int getCaretPosition() {
            return JTextComponent.this.getCaretPosition();
        }
        
        @Override
        public AttributeSet getCharacterAttribute(int n) {
            Element element = null;
            if ((JTextComponent.this.model instanceof AbstractDocument)) {
                ((AbstractDocument)JTextComponent.this.model).readLock();
            }
            try {
                for (element = JTextComponent.this.model.getDefaultRootElement(); !element.isLeaf(); element = element.getElement(element.getElementIndex(n))) {}
            }
            finally {
                if ((JTextComponent.this.model instanceof AbstractDocument)) {
                    ((AbstractDocument)JTextComponent.this.model).readUnlock();
                }
            }
            return element.getAttributes();
        }
        
        @Override
        public int getSelectionStart() {
            return JTextComponent.this.getSelectionStart();
        }
        
        @Override
        public int getSelectionEnd() {
            return JTextComponent.this.getSelectionEnd();
        }
        
        @Override
        public String getSelectedText() {
            return JTextComponent.this.getSelectedText();
        }
        
        @Override
        public String getAtIndex(int n, int n2) {
            return this.getAtIndex(n, n2, 0);
        }
        
        @Override
        public String getAfterIndex(int n, int n2) {
            return this.getAtIndex(n, n2, 1);
        }
        
        @Override
        public String getBeforeIndex(int n, int n2) {
            return this.getAtIndex(n, n2, -1);
        }
        
        private String getAtIndex(int n, int n2, int n3) {
            if ((JTextComponent.this.model instanceof AbstractDocument)) {
                ((AbstractDocument)JTextComponent.this.model).readLock();
            }
            try {
                if (n2 < 0 || n2 >= JTextComponent.this.model.getLength()) {
                    __Null _Null = null;
                }
                else {
                    switch (n) {
                        case 1: {
                            if (n2 + n3 < JTextComponent.this.model.getLength() && n2 + n3 >= 0) {
                                String text = JTextComponent.this.model.getText(n2 + n3, 1);
                            }
                            else {
                                break;
                            }
                        }
                        case 2:
                        case 3: {
                            JTextComponent.AccessibleJTextComponent.IndexedSegment indexedSegment = this.getSegmentAt(n, n2);
                            if (indexedSegment != null) {
                                if (n3 != 0) {
                                    int n4;
                                    if (n3 < 0) {
                                        n4 = indexedSegment.modelOffset - 1;
                                    }
                                    else {
                                        n4 = indexedSegment.modelOffset + n3 * indexedSegment.count;
                                    }
                                    if (n4 >= 0 && n4 <= JTextComponent.this.model.getLength()) {
                                        indexedSegment = this.getSegmentAt(n, n4);
                                    }
                                    else {
                                        indexedSegment = null;
                                    }
                                }
                                if (indexedSegment != null) {
                                    String o = new String(indexedSegment.array, indexedSegment.offset, indexedSegment.count);
                                }
                                else {
                                    break;
                                }
                            }
                            else {
                                break;
                            }
                        }
                    }
                }
            }
            catch (BadLocationException ex) {}
            finally {
                if ((JTextComponent.this.model instanceof AbstractDocument)) {
                    ((AbstractDocument)JTextComponent.this.model).readUnlock();
                }
            }
            return null;
        }
        
        private Element getParagraphElement(int n) {
            if ((JTextComponent.this.model instanceof PlainDocument)) {
                return ((PlainDocument)JTextComponent.this.model).getParagraphElement(n);
            }
            if ((JTextComponent.this.model instanceof StyledDocument)) {
                return ((StyledDocument)JTextComponent.this.model).getParagraphElement(n);
            }
            Element element;
            for (element = JTextComponent.this.model.getDefaultRootElement(); !element.isLeaf(); element = element.getElement(element.getElementIndex(n))) {}
            if (element == null) {
                return null;
            }
            return element.getParentElement();
        }
        
        private JTextComponent.AccessibleJTextComponent.IndexedSegment getParagraphElementText(int n) throws BadLocationException {
            Element paragraphElement = this.getParagraphElement(n);
            if (paragraphElement != null) {
                JTextComponent.AccessibleJTextComponent.IndexedSegment indexedSegment = new JTextComponent.AccessibleJTextComponent.IndexedSegment();
                try {
                    JTextComponent.this.model.getText(paragraphElement.getStartOffset(), paragraphElement.getEndOffset() - paragraphElement.getStartOffset(), indexedSegment);
                }
                catch (BadLocationException ex) {
                    return null;
                }
                indexedSegment.modelOffset = paragraphElement.getStartOffset();
                return indexedSegment;
            }
            return null;
        }
        
        private JTextComponent.AccessibleJTextComponent.IndexedSegment getSegmentAt(int n, int n2) throws BadLocationException {
            JTextComponent.AccessibleJTextComponent.IndexedSegment paragraphElementText = this.getParagraphElementText(n2);
            if (paragraphElementText == null) {
                return null;
            }
            BreakIterator breakIterator;
            switch (n) {
                case 2: {
                    breakIterator = BreakIterator.getWordInstance(this.getLocale());
                    break;
                }
                case 3: {
                    breakIterator = BreakIterator.getSentenceInstance(this.getLocale());
                    break;
                }
                default: {
                    return null;
                }
            }
            paragraphElementText.first();
            breakIterator.setText(paragraphElementText);
            int following = breakIterator.following(n2 - paragraphElementText.modelOffset + paragraphElementText.offset);
            if (following == -1) {
                return null;
            }
            if (following > paragraphElementText.offset + paragraphElementText.count) {
                return null;
            }
            int previous = breakIterator.previous();
            if (previous == -1 || previous >= paragraphElementText.offset + paragraphElementText.count) {
                return null;
            }
            paragraphElementText.modelOffset = paragraphElementText.modelOffset + previous - paragraphElementText.offset;
            paragraphElementText.offset = previous;
            paragraphElementText.count = following - previous;
            return paragraphElementText;
        }
        
        @Override
        public AccessibleEditableText getAccessibleEditableText() {
            return this;
        }
        
        @Override
        public void setTextContents(String text) {
            JTextComponent.this.setText(text);
        }
        
        @Override
        public void insertTextAtIndex(int n, String s) {
            Document document = JTextComponent.this.getDocument();
            if (document != null) {
                try {
                    if (s != null && s.length() > 0) {
                        int saveComposedText = JTextComponent.this.saveComposedText(n);
                        document.insertString(n, s, null);
                        if (!(saveComposedText == 0)) {
                            JTextComponent.this.restoreComposedText();
                        }
                    }
                }
                catch (BadLocationException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(JTextComponent.this);
                }
            }
        }
        
        @Override
        public String getTextRange(int n, int n2) {
            String text = null;
            int min = Math.min(n, n2);
            int max = Math.max(n, n2);
            if (min != max) {
                try {
                    text = JTextComponent.this.getDocument().getText(min, max - min);
                }
                catch (BadLocationException ex) {
                    throw new IllegalArgumentException(ex.getMessage());
                }
            }
            return text;
        }
        
        @Override
        public void delete(int n, int n2) {
            if (JTextComponent.this.isEditable() && this.isEnabled()) {
                try {
                    int min = Math.min(n, n2);
                    int max = Math.max(n, n2);
                    if (min != max) {
                        JTextComponent.this.getDocument().remove(min, max - min);
                    }
                }
                catch (BadLocationException ex) {}
            }
            else {
                UIManager.getLookAndFeel().provideErrorFeedback(JTextComponent.this);
            }
        }
        
        @Override
        public void cut(int n, int n2) {
            this.selectText(n, n2);
            JTextComponent.this.cut();
        }
        
        @Override
        public void paste(int caretPosition) {
            JTextComponent.this.setCaretPosition(caretPosition);
            JTextComponent.this.paste();
        }
        
        @Override
        public void replaceText(int n, int n2, String s) {
            this.selectText(n, n2);
            JTextComponent.this.replaceSelection(s);
        }
        
        @Override
        public void selectText(int n, int n2) {
            JTextComponent.this.select(n, n2);
        }
        
        @Override
        public void setAttributes(int n, int n2, AttributeSet set) {
            Document document = JTextComponent.this.getDocument();
            if (document != null && document instanceof StyledDocument) {
                StyledDocument styledDocument = (StyledDocument)document;
                styledDocument.setCharacterAttributes(n, n2 - n, set, true);
            }
        }
        
        private AccessibleTextSequence getSequenceAtIndex(int n, int runEdge, int detailMessage) {
            if (runEdge < 0 || runEdge >= JTextComponent.this.model.getLength()) {
                return null;
            }
            if (detailMessage < -1 || detailMessage > 1) {
                return null;
            }
            switch (n) {
                case 1: {
                    if ((JTextComponent.this.model instanceof AbstractDocument)) {
                        ((AbstractDocument)JTextComponent.this.model).readLock();
                    }
                    AccessibleTextSequence accessibleTextSequence = null;
                    try {
                        if (runEdge + detailMessage < JTextComponent.this.model.getLength() && runEdge + detailMessage >= 0) {
                            accessibleTextSequence = new AccessibleTextSequence(runEdge + detailMessage, runEdge + detailMessage + 1, JTextComponent.this.model.getText(runEdge + detailMessage, 1));
                        }
                    }
                    catch (BadLocationException ex) {}
                    finally {
                        if ((JTextComponent.this.model instanceof AbstractDocument)) {
                            ((AbstractDocument)JTextComponent.this.model).readUnlock();
                        }
                    }
                    return accessibleTextSequence;
                }
                case 2:
                case 3: {
                    if ((JTextComponent.this.model instanceof AbstractDocument)) {
                        ((AbstractDocument)JTextComponent.this.model).readLock();
                    }
                    Object o = null;
                    try {
                        JTextComponent.AccessibleJTextComponent.IndexedSegment indexedSegment = this.getSegmentAt(n, runEdge);
                        if (indexedSegment != null) {
                            if (detailMessage != 0) {
                                int n2;
                                if (detailMessage < 0) {
                                    n2 = indexedSegment.modelOffset - 1;
                                }
                                else {
                                    n2 = indexedSegment.modelOffset + indexedSegment.count;
                                }
                                if (n2 >= 0 && n2 <= JTextComponent.this.model.getLength()) {
                                    indexedSegment = this.getSegmentAt(n, n2);
                                }
                                else {
                                    indexedSegment = null;
                                }
                            }
                            if (indexedSegment != null && indexedSegment.offset + indexedSegment.count <= JTextComponent.this.model.getLength()) {
                                o = new AccessibleTextSequence(indexedSegment.offset, indexedSegment.offset + indexedSegment.count, new String(indexedSegment.array, indexedSegment.offset, indexedSegment.count));
                            }
                        }
                    }
                    catch (BadLocationException ex2) {}
                    finally {
                        if ((JTextComponent.this.model instanceof AbstractDocument)) {
                            ((AbstractDocument)JTextComponent.this.model).readUnlock();
                        }
                    }
                    return o;
                }
                case 4: {
                    AccessibleTextSequence accessibleTextSequence2 = null;
                    if ((JTextComponent.this.model instanceof AbstractDocument)) {
                        ((AbstractDocument)JTextComponent.this.model).readLock();
                    }
                    try {
                        int rowStart = Utilities.getRowStart(JTextComponent.this, runEdge);
                        int rowEnd = Utilities.getRowEnd(JTextComponent.this, runEdge);
                        if (rowStart >= 0 && rowEnd >= rowStart) {
                            if (detailMessage == 0) {
                                accessibleTextSequence2 = new AccessibleTextSequence(rowStart, rowEnd, JTextComponent.this.model.getText(rowStart, rowEnd - rowStart + 1));
                            }
                            else if (detailMessage == -1 && rowStart > 0) {
                                int rowEnd3 = Utilities.getRowEnd(JTextComponent.this, rowStart - 1);
                                int rowStart3 = Utilities.getRowStart(JTextComponent.this, rowStart - 1);
                                if (rowStart3 >= 0 && rowEnd3 >= rowStart3) {
                                    accessibleTextSequence2 = new AccessibleTextSequence(rowStart3, rowEnd3, JTextComponent.this.model.getText(rowStart3, rowEnd3 - rowStart3 + 1));
                                }
                            }
                            else if (detailMessage == 1 && rowEnd < JTextComponent.this.model.getLength()) {
                                int rowStart2 = Utilities.getRowStart(JTextComponent.this, rowEnd + 1);
                                int rowEnd2 = Utilities.getRowEnd(JTextComponent.this, rowEnd + 1);
                                if (rowStart2 >= 0 && rowEnd2 >= rowStart2) {
                                    accessibleTextSequence2 = new AccessibleTextSequence(rowStart2, rowEnd2, JTextComponent.this.model.getText(rowStart2, rowEnd2 - rowStart2 + 1));
                                }
                            }
                        }
                    }
                    catch (BadLocationException ex3) {}
                    finally {
                        if ((JTextComponent.this.model instanceof AbstractDocument)) {
                            ((AbstractDocument)JTextComponent.this.model).readUnlock();
                        }
                    }
                    return accessibleTextSequence2;
                }
                case 5: {
                    String text = null;
                    if ((JTextComponent.this.model instanceof AbstractDocument)) {
                        ((AbstractDocument)JTextComponent.this.model).readLock();
                    }
                    int start;
                    int runEdge2;
                    try {
                        runEdge2 = (start = Integer.MIN_VALUE);
                        switch (detailMessage) {
                            case -1: {
                                runEdge2 = this.getRunEdge(runEdge, detailMessage);
                                runEdge = runEdge2 - 1;
                                break;
                            }
                            case 1: {
                                start = (runEdge = this.getRunEdge(runEdge, detailMessage));
                                break;
                            }
                            case 0: {
                                break;
                            }
                            default: {
                                throw new AssertionError(detailMessage);
                            }
                        }
                        start = ((start != Integer.MIN_VALUE) ? start : this.getRunEdge(runEdge, -1));
                        runEdge2 = ((runEdge2 != Integer.MIN_VALUE) ? runEdge2 : this.getRunEdge(runEdge, 1));
                        text = JTextComponent.this.model.getText(start, runEdge2 - start);
                    }
                    catch (BadLocationException ex4) {
                        __Null _Null = null;
                    }
                    finally {
                        if ((JTextComponent.this.model instanceof AbstractDocument)) {
                            ((AbstractDocument)JTextComponent.this.model).readUnlock();
                        }
                    }
                    return new AccessibleTextSequence(start, runEdge2, text);
                }
                default: {
                    return null;
                }
            }
        }
        
        private int getRunEdge(int detailMessage, int detailMessage2) throws BadLocationException {
            if (detailMessage < 0 || detailMessage >= JTextComponent.this.model.getLength()) {
                throw new BadLocationException("Location out of bounds", detailMessage);
            }
            int elementIndex;
            Element element;
            for (elementIndex = -1, element = JTextComponent.this.model.getDefaultRootElement(); !element.isLeaf(); element = element.getElement(elementIndex)) {
                elementIndex = element.getElementIndex(detailMessage);
            }
            if (elementIndex == -1) {
                throw new AssertionError(detailMessage);
            }
            AttributeSet attributes = element.getAttributes();
            Element parentElement = element.getParentElement();
            switch (detailMessage2) {
                case -1:
                case 1: {
                    for (int elementCount = parentElement.getElementCount(); elementIndex + detailMessage2 > 0 && elementIndex + detailMessage2 < elementCount && parentElement.getElement(elementIndex + detailMessage2).getAttributes().isEqual(attributes); elementIndex += detailMessage2) {}
                    Element element2 = parentElement.getElement(elementIndex);
                    switch (detailMessage2) {
                        case -1: {
                            return element2.getStartOffset();
                        }
                        case 1: {
                            return element2.getEndOffset();
                        }
                        default: {
                            return Integer.MIN_VALUE;
                        }
                    }
                }
                default: {
                    throw new AssertionError(detailMessage2);
                }
            }
        }
        
        @Override
        public AccessibleTextSequence getTextSequenceAt(int n, int n2) {
            return this.getSequenceAtIndex(n, n2, 0);
        }
        
        @Override
        public AccessibleTextSequence getTextSequenceAfter(int n, int n2) {
            return this.getSequenceAtIndex(n, n2, 1);
        }
        
        @Override
        public AccessibleTextSequence getTextSequenceBefore(int n, int n2) {
            return this.getSequenceAtIndex(n, n2, -1);
        }
        
        @Override
        public Rectangle getTextBounds(int n, int n2) {
            if (n < 0 || n > JTextComponent.this.model.getLength() - 1 || n2 < 0 || n2 > JTextComponent.this.model.getLength() - 1 || n > n2) {
                return null;
            }
            TextUI ui = JTextComponent.this.getUI();
            if (ui == null) {
                return null;
            }
            Rectangle rectangle = null;
            Rectangle rootEditorRect = this.getRootEditorRect();
            if (rootEditorRect == null) {
                return null;
            }
            if ((JTextComponent.this.model instanceof AbstractDocument)) {
                ((AbstractDocument)JTextComponent.this.model).readLock();
            }
            try {
                View rootView = ui.getRootView(JTextComponent.this);
                if (rootView != null) {
                    Shape modelToView = rootView.modelToView(n, Position.Bias.Forward, n2, Position.Bias.Backward, rootEditorRect);
                    rectangle = ((modelToView instanceof Rectangle) ? ((Rectangle)modelToView) : modelToView.getBounds());
                }
            }
            catch (BadLocationException ex) {}
            finally {
                if ((JTextComponent.this.model instanceof AbstractDocument)) {
                    ((AbstractDocument)JTextComponent.this.model).readUnlock();
                }
            }
            return rectangle;
        }
        
        @Override
        public AccessibleAction getAccessibleAction() {
            return this;
        }
        
        @Override
        public int getAccessibleActionCount() {
            return JTextComponent.this.getActions().length;
        }
        
        @Override
        public String getAccessibleActionDescription(int n) {
            Action[] actions = JTextComponent.this.getActions();
            if (n < 0 || n >= actions.length) {
                return null;
            }
            return (String)actions[n].getValue("Name");
        }
        
        @Override
        public boolean doAccessibleAction(int n) {
            Action[] actions = JTextComponent.this.getActions();
            if (n < 0 || n >= actions.length) {
                return false;
            }
            actions[n].actionPerformed(new ActionEvent(JTextComponent.this, 1001, null, EventQueue.getMostRecentEventTime(), JTextComponent.access$300(JTextComponent.this)));
            return true;
        }
        
        private class IndexedSegment extends Segment
        {
            public int modelOffset;
        }
    }
    
    public static final class DropLocation extends TransferHandler.DropLocation
    {
        private final int index;
        private final Position.Bias bias;
        
        private DropLocation(Point dropPoint, int index, Position.Bias bias) {
            super(dropPoint);
            this.index = index;
            this.bias = bias;
        }
        
        public int getIndex() {
            return this.index;
        }
        
        public Position.Bias getBias() {
            return this.bias;
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "[dropPoint=" + this.getDropPoint() + ",index=" + this.index + ",bias=" + this.bias + "]";
        }
    }
    
    static class DefaultTransferHandler extends TransferHandler implements UIResource
    {
        @Override
        public void exportToClipboard(JComponent component, Clipboard clipboard, int n) throws IllegalStateException {
            if ((component instanceof JTextComponent)) {
                JTextComponent textComponent = (JTextComponent)component;
                int selectionStart = textComponent.getSelectionStart();
                int selectionEnd = textComponent.getSelectionEnd();
                if (selectionStart != selectionEnd) {
                    try {
                        Document document = textComponent.getDocument();
                        clipboard.setContents(new StringSelection(document.getText(selectionStart, selectionEnd - selectionStart)), null);
                        if (n == 2) {
                            document.remove(selectionStart, selectionEnd - selectionStart);
                        }
                    }
                    catch (BadLocationException ex) {}
                }
            }
        }
        
        @Override
        public boolean importData(JComponent component, Transferable transferable) {
            if ((component instanceof JTextComponent)) {
                DataFlavor flavor = this.getFlavor(transferable.getTransferDataFlavors());
                if (flavor != null) {
                    InputContext inputContext = component.getInputContext();
                    if (inputContext != null) {
                        inputContext.endComposition();
                    }
                    try {
                        ((JTextComponent)component).replaceSelection((String)transferable.getTransferData(flavor));
                        return true;
                    }
                    catch (UnsupportedFlavorException ex) {}
                    catch (IOException ex2) {}
                }
            }
            return false;
        }
        
        @Override
        public boolean canImport(JComponent component, DataFlavor[] array) {
            JTextComponent textComponent = (JTextComponent)component;
            return textComponent.isEditable() && textComponent.isEnabled() && this.getFlavor(array) != null;
        }
        
        @Override
        public int getSourceActions(JComponent component) {
            return 0;
        }
        
        private DataFlavor getFlavor(DataFlavor[] array) {
            if (array != null) {
                int length = array.length;
                int i = 0;
                while (i < length) {
                    DataFlavor dataFlavor = array[i];
                    if (dataFlavor.equals(DataFlavor.stringFlavor)) {
                        return dataFlavor;
                    }
                    ++i;
                }
            }
            return null;
        }
    }
    
    static class DefaultKeymap implements Keymap
    {
        String nm;
        Keymap parent;
        Hashtable<KeyStroke, Action> bindings;
        Action defaultAction;
        
        DefaultKeymap(String nm, Keymap parent) {
            this.nm = nm;
            this.parent = parent;
            this.bindings = new Hashtable();
        }
        
        @Override
        public Action getDefaultAction() {
            if (this.defaultAction != null) {
                return this.defaultAction;
            }
            return (this.parent != null) ? this.parent.getDefaultAction() : null;
        }
        
        @Override
        public void setDefaultAction(Action defaultAction) {
            this.defaultAction = defaultAction;
        }
        
        @Override
        public String getName() {
            return this.nm;
        }
        
        @Override
        public Action getAction(KeyStroke key) {
            Action action = (Action)this.bindings.get(key);
            if (action == null && this.parent != null) {
                action = this.parent.getAction(key);
                return action;
            }
            return action;
        }
        
        @Override
        public KeyStroke[] getBoundKeyStrokes() {
            KeyStroke[] array = new KeyStroke[this.bindings.size()];
            int n = 0;
            Enumeration<KeyStroke> keys = this.bindings.keys();
            while (keys.hasMoreElements()) {
                array[n++] = (KeyStroke)keys.nextElement();
            }
            return array;
        }
        
        @Override
        public Action[] getBoundActions() {
            Action[] array = new Action[this.bindings.size()];
            int n = 0;
            Enumeration<Action> elements = this.bindings.elements();
            while (elements.hasMoreElements()) {
                array[n++] = (Action)elements.nextElement();
            }
            return array;
        }
        
        @Override
        public KeyStroke[] getKeyStrokesForAction(Action action) {
            if (action == null) {
                return null;
            }
            KeyStroke[] array = null;
            Object o = null;
            Enumeration<KeyStroke> keys = this.bindings.keys();
            while (keys.hasMoreElements()) {
                KeyStroke keyStroke = (KeyStroke)keys.nextElement();
                if (this.bindings.get(keyStroke) == action) {
                    if (o == null) {
                        o = new Vector();
                    }
                    o.addElement(keyStroke);
                }
                else {
                    continue;
                }
            }
            if (this.parent != null) {
                KeyStroke[] keyStrokesForAction = this.parent.getKeyStrokesForAction(action);
                if (keyStrokesForAction != null) {
                    int n = 0;
                    for (int i = keyStrokesForAction.length - 1; i >= 0; --i) {
                        if (this.isLocallyDefined(keyStrokesForAction[i])) {
                            keyStrokesForAction[i] = null;
                            ++n;
                        }
                    }
                    if (n > 0 && n < keyStrokesForAction.length) {
                        if (o == null) {
                            o = new Vector();
                        }
                        for (int j = keyStrokesForAction.length - 1; j >= 0; --j) {
                            if (keyStrokesForAction[j] != null) {
                                o.addElement(keyStrokesForAction[j]);
                            }
                        }
                    }
                    else if (n == 0) {
                        if (o == null) {
                            array = keyStrokesForAction;
                        }
                        else {
                            array = new KeyStroke[o.size() + keyStrokesForAction.length];
                            o.copyInto(array);
                            System.arraycopy(keyStrokesForAction, 0, array, o.size(), keyStrokesForAction.length);
                            o = null;
                        }
                    }
                }
            }
            if (o != null) {
                array = new KeyStroke[o.size()];
                o.copyInto(array);
                return array;
            }
            return array;
        }
        
        @Override
        public boolean isLocallyDefined(KeyStroke key) {
            return this.bindings.containsKey(key);
        }
        
        @Override
        public void addActionForKeyStroke(KeyStroke key, Action value) {
            this.bindings.put(key, value);
        }
        
        @Override
        public void removeKeyStrokeBinding(KeyStroke key) {
            this.bindings.remove(key);
        }
        
        @Override
        public void removeBindings() {
            this.bindings.clear();
        }
        
        @Override
        public Keymap getResolveParent() {
            return this.parent;
        }
        
        @Override
        public void setResolveParent(Keymap parent) {
            this.parent = parent;
        }
        
        @Override
        public String toString() {
            return "Keymap[" + this.nm + "]" + this.bindings;
        }
    }
    
    static class KeymapWrapper extends InputMap
    {
        static final Object DefaultActionKey;
        private Keymap keymap;
        
        KeymapWrapper(Keymap keymap) {
            this.keymap = keymap;
        }
        
        @Override
        public KeyStroke[] keys() {
            KeyStroke[] keys = super.keys();
            KeyStroke[] boundKeyStrokes = this.keymap.getBoundKeyStrokes();
            int n = (keys == null) ? 0 : keys.length;
            int n2 = (boundKeyStrokes == null) ? 0 : boundKeyStrokes.length;
            if (n == 0) {
                return boundKeyStrokes;
            }
            if (n2 == 0) {
                return keys;
            }
            KeyStroke[] array = new KeyStroke[n + n2];
            System.arraycopy(keys, 0, array, 0, n);
            System.arraycopy(boundKeyStrokes, 0, array, n, n2);
            return array;
        }
        
        @Override
        public int size() {
            KeyStroke[] boundKeyStrokes = this.keymap.getBoundKeyStrokes();
            return super.size() + ((boundKeyStrokes == null) ? 0 : boundKeyStrokes.length);
        }
        
        @Override
        public Object get(KeyStroke keyStroke) {
            Action action = this.keymap.getAction(keyStroke);
            if (action == null) {
                action = super.get(keyStroke);
                if (action == null && keyStroke.getKeyChar() != '\uffff' && this.keymap.getDefaultAction() != null) {
                    action = JTextComponent.KeymapWrapper.DefaultActionKey;
                }
            }
            return action;
        }
        
        static {
            DefaultActionKey = new Object();
        }
    }
    
    static class KeymapActionMap extends ActionMap
    {
        private Keymap keymap;
        
        KeymapActionMap(Keymap keymap) {
            this.keymap = keymap;
        }
        
        @Override
        public Object[] keys() {
            Object[] keys = super.keys();
            Action[] boundActions = this.keymap.getBoundActions();
            int n = (keys == null) ? 0 : keys.length;
            int n2 = (boundActions == null) ? 0 : boundActions.length;
            int n3 = this.keymap.getDefaultAction() != null;
            if (!(n3 == 0)) {
                ++n2;
            }
            if (n == 0) {
                if (!(n3 == 0)) {
                    Object[] array2 = new Object[n2];
                    if (n2 > 1) {
                        System.arraycopy(boundActions, 0, array2, 0, n2 - 1);
                    }
                    array2[n2 - 1] = JTextComponent.KeymapWrapper.DefaultActionKey;
                    return array2;
                }
                else {
                    return boundActions;
                }
            }
            else {
                if (n2 == 0) {
                    return keys;
                }
                Object[] array = new Object[n + n2];
                System.arraycopy(keys, 0, array, 0, n);
                if (!(n3 == 0)) {
                    if (n2 > 1) {
                        System.arraycopy(boundActions, 0, array, n, n2 - 1);
                    }
                    array[n + n2 - 1] = JTextComponent.KeymapWrapper.DefaultActionKey;
                    return array;
                }
                else {
                    System.arraycopy(boundActions, 0, array, n, n2);
                    return array;
                }
            }
        }
        
        @Override
        public int size() {
            Action[] boundActions = this.keymap.getBoundActions();
            int n = (boundActions == null) ? 0 : boundActions.length;
            if (this.keymap.getDefaultAction() != null) {
                ++n;
            }
            return super.size() + n;
        }
        
        @Override
        public Action get(Object key) {
            Action action = super.get(key);
            if (action == null) {
                if (key == JTextComponent.KeymapWrapper.DefaultActionKey) {
                    action = this.keymap.getDefaultAction();
                }
                else if ((key instanceof Action)) {
                    action = (Action)key;
                }
            }
            return action;
        }
    }
    
    static class MutableCaretEvent extends CaretEvent implements ChangeListener, FocusListener, MouseListener
    {
        private boolean dragActive;
        private int dot;
        private int mark;
        
        MutableCaretEvent(JTextComponent source) {
            super(source);
        }
        
        final void fire() {
            JTextComponent textComponent = (JTextComponent)this.getSource();
            if (textComponent != null) {
                Caret caret = textComponent.getCaret();
                this.dot = caret.getDot();
                this.mark = caret.getMark();
                textComponent.fireCaretUpdate(this);
            }
        }
        
        @Override
        public final String toString() {
            return "dot=" + this.dot + ",mark=" + this.mark;
        }
        
        @Override
        public final int getDot() {
            return this.dot;
        }
        
        @Override
        public final int getMark() {
            return this.mark;
        }
        
        @Override
        public final void stateChanged(ChangeEvent changeEvent) {
            if (!this.dragActive) {
                this.fire();
            }
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            AppContext.getAppContext().put(JTextComponent.FOCUSED_COMPONENT, focusEvent.getSource());
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
        }
        
        @Override
        public final void mousePressed(MouseEvent mouseEvent) {
            this.dragActive = true;
        }
        
        @Override
        public final void mouseReleased(MouseEvent mouseEvent) {
            this.dragActive = false;
            this.fire();
        }
        
        @Override
        public final void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public final void mouseEntered(MouseEvent mouseEvent) {
        }
        
        @Override
        public final void mouseExited(MouseEvent mouseEvent) {
        }
    }
    
    class InputMethodRequestsHandler implements InputMethodRequests, DocumentListener
    {
        @Override
        public AttributedCharacterIterator cancelLatestCommittedText(AttributedCharacterIterator.Attribute[] array) {
            Document document = JTextComponent.this.getDocument();
            if (document != null && JTextComponent.this.latestCommittedTextStart != null && !JTextComponent.this.latestCommittedTextStart.equals(JTextComponent.this.latestCommittedTextEnd)) {
                try {
                    int offset = JTextComponent.this.latestCommittedTextStart.getOffset();
                    int offset2 = JTextComponent.this.latestCommittedTextEnd.getOffset();
                    String text = document.getText(offset, offset2 - offset);
                    document.remove(offset, offset2 - offset);
                    return new AttributedString(text).getIterator();
                }
                catch (BadLocationException ex) {}
            }
            return null;
        }
        
        @Override
        public AttributedCharacterIterator getCommittedText(int n, int n2, AttributedCharacterIterator.Attribute[] array) {
            int offset = 0;
            int offset2 = 0;
            if (JTextComponent.this.composedTextExists()) {
                offset = JTextComponent.this.composedTextStart.getOffset();
                offset2 = JTextComponent.this.composedTextEnd.getOffset();
            }
            String text;
            try {
                if (n < offset) {
                    if (n2 <= offset) {
                        text = JTextComponent.this.getText(n, n2 - n);
                    }
                    else {
                        int n3 = offset - n;
                        text = JTextComponent.this.getText(n, n3) + JTextComponent.this.getText(offset2, n2 - n - n3);
                    }
                }
                else {
                    text = JTextComponent.this.getText(n + (offset2 - offset), n2 - n);
                }
            }
            catch (BadLocationException ex) {
                throw new IllegalArgumentException("Invalid range");
            }
            return new AttributedString(text).getIterator();
        }
        
        @Override
        public int getCommittedTextLength() {
            Document document = JTextComponent.this.getDocument();
            int length = 0;
            if (document != null) {
                length = document.getLength();
                if (JTextComponent.this.composedTextContent != null) {
                    if (JTextComponent.this.composedTextEnd == null || JTextComponent.this.composedTextStart == null) {
                        length -= JTextComponent.this.composedTextContent.length();
                    }
                    else {
                        length -= JTextComponent.this.composedTextEnd.getOffset() - JTextComponent.this.composedTextStart.getOffset();
                    }
                }
            }
            return length;
        }
        
        @Override
        public int getInsertPositionOffset() {
            int offset = 0;
            int offset2 = 0;
            if (JTextComponent.this.composedTextExists()) {
                offset = JTextComponent.this.composedTextStart.getOffset();
                offset2 = JTextComponent.this.composedTextEnd.getOffset();
            }
            int caretPosition = JTextComponent.this.getCaretPosition();
            if (caretPosition < offset) {
                return caretPosition;
            }
            if (caretPosition < offset2) {
                return offset;
            }
            return caretPosition - (offset2 - offset);
        }
        
        @Override
        public TextHitInfo getLocationOffset(int n, int n2) {
            if (JTextComponent.this.composedTextAttribute == null) {
                return null;
            }
            Point locationOnScreen = JTextComponent.this.getLocationOnScreen();
            locationOnScreen.x = n - locationOnScreen.x;
            locationOnScreen.y = n2 - locationOnScreen.y;
            int viewToModel = JTextComponent.this.viewToModel(locationOnScreen);
            if (viewToModel >= JTextComponent.this.composedTextStart.getOffset() && viewToModel <= JTextComponent.this.composedTextEnd.getOffset()) {
                return TextHitInfo.leading(viewToModel - JTextComponent.this.composedTextStart.getOffset());
            }
            return null;
        }
        
        @Override
        public Rectangle getTextLocation(TextHitInfo textHitInfo) {
            Object modelToView;
            try {
                modelToView = JTextComponent.this.modelToView(JTextComponent.this.getCaretPosition());
                if (modelToView != null) {
                    Point locationOnScreen = JTextComponent.this.getLocationOnScreen();
                    modelToView.translate(locationOnScreen.x, locationOnScreen.y);
                }
            }
            catch (BadLocationException ex) {
                modelToView = null;
            }
            if (modelToView == null) {
                modelToView = new Rectangle();
                return modelToView;
            }
            return modelToView;
        }
        
        @Override
        public AttributedCharacterIterator getSelectedText(AttributedCharacterIterator.Attribute[] array) {
            String selectedText = JTextComponent.this.getSelectedText();
            if (selectedText != null) {
                return new AttributedString(selectedText).getIterator();
            }
            return null;
        }
        
        @Override
        public void changedUpdate(DocumentEvent documentEvent) {
            JTextComponent.this.latestCommittedTextStart = (JTextComponent.this.latestCommittedTextEnd = null);
        }
        
        @Override
        public void insertUpdate(DocumentEvent documentEvent) {
            JTextComponent.this.latestCommittedTextStart = (JTextComponent.this.latestCommittedTextEnd = null);
        }
        
        @Override
        public void removeUpdate(DocumentEvent documentEvent) {
            JTextComponent.this.latestCommittedTextStart = (JTextComponent.this.latestCommittedTextEnd = null);
        }
    }
    
    class ComposedTextCaret extends DefaultCaret implements Serializable
    {
        Color bg;
        
        @Override
        public void install(JTextComponent textComponent) {
            super.install(textComponent);
            Document document = textComponent.getDocument();
            if ((document instanceof StyledDocument)) {
                StyledDocument styledDocument = (StyledDocument)document;
                this.bg = styledDocument.getBackground(styledDocument.getCharacterElement(textComponent.composedTextStart.getOffset()).getAttributes());
            }
            if (this.bg == null) {
                this.bg = textComponent.getBackground();
            }
        }
        
        @Override
        public void paint(Graphics graphics) {
            if (this.isVisible()) {
                try {
                    Rectangle modelToView = this.component.modelToView(this.getDot());
                    graphics.setXORMode(this.bg);
                    graphics.drawLine(modelToView.x, modelToView.y, modelToView.x, modelToView.y + modelToView.height - 1);
                    graphics.setPaintMode();
                }
                catch (BadLocationException ex) {}
            }
        }
        
        @Override
        protected void positionCaret(MouseEvent mouseEvent) {
            JTextComponent component = this.component;
            int viewToModel = component.viewToModel(new Point(mouseEvent.getX(), mouseEvent.getY()));
            if (viewToModel >= component.composedTextStart.getOffset() && viewToModel <= JTextComponent.this.composedTextEnd.getOffset()) {
                super.positionCaret(mouseEvent);
                return;
            }
            try {
                Position position = component.getDocument().createPosition(viewToModel);
                component.getInputContext().endComposition();
                EventQueue.invokeLater(new JTextComponent.DoSetCaretPosition(component, position));
            }
            catch (BadLocationException x) {
                System.err.println(x);
            }
        }
    }
    
    private class DoSetCaretPosition implements Runnable
    {
        JTextComponent host;
        Position newPos;
        
        DoSetCaretPosition(JTextComponent host, Position newPos) {
            this.host = host;
            this.newPos = newPos;
        }
        
        @Override
        public void run() {
            this.host.setCaretPosition(this.newPos.getOffset());
        }
    }
}
